searchNodes=[{"doc":"Methods for interacting with ArangoDB that aren't through Ecto. This allows for easy interaction with graph functions of Arango. Using Ecto's relations for edge relations was tried but it was found to be too much of a 'hacky' solution. Using separate functions that still utilise Ecto document queries was found to be the optimal solution.","ref":"ArangoXEcto.html","title":"ArangoXEcto","type":"module"},{"doc":"Runs an Arangox function using a repo This is simply a helper function that extracts the connection from the repo and runs a regular query. Parameters repo - The Ecto repo module used for connection function - An atom of the Arangox function to run args - The options passed to the function (not including the conn argument) The conn argument is automatically prepended to your supplied args Supported Functions :abort :cursor :delete :delete! :get :get! :head :head! :options :options! :patch :patch! :post :post! :put :put! :request :request! :run :status :transaction (use built in Ecto.Repo.transaction/2 instead) Examples iex&gt; ArangoXEcto . api_query ( Repo , :get , [ &quot;/_api/collection&quot; ] ) { :ok , % Arangox.Response { body : ... } } iex&gt; ArangoXEcto . api_query ( Repo , :non_existent , [ &quot;/_api/collection&quot; ] ) ** (ArgumentError) Invalid function passed to `Arangox` module","ref":"ArangoXEcto.html#api_query/3","title":"ArangoXEcto.api_query/3","type":"function"},{"doc":"Runs a raw AQL query on the database. This will create a transaction and cursor on Arango and run the raw query. If there is an error in the query such as a syntax error, an Arangox.Error will be raised. Parameters repo - The Ecto repo module to use for queries query - The AQL query string to execute vars - A keyword list or a map with the values for variables in the query opts - Options to be passed to DBConnection.transaction/3 Examples iex&gt; ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] }","ref":"ArangoXEcto.html#aql_query/4","title":"ArangoXEcto.aql_query/4","type":"function"},{"doc":"Checks if a collection exists This will return true if the collection exists in the database, matches the specified type and is not a system database, otherwise it will be false. Parameters repo - The Ecto repo module to use for the query collection_name - Name of the collection to check type - The type of collection to check against, defaults to a regular document Examples Checking a document collection exists iex&gt; ArangoXEcto . collection_exists? ( Repo , :users ) true Checking an edge collection exists iex&gt; ArangoXEcto . collection_exists? ( Repo , &quot;my_edge&quot; , :edge ) true Checking a system document collection exists does not work iex&gt; ArangoXEcto . collection_exists? ( Repo , &quot;_system_test&quot; ) false","ref":"ArangoXEcto.html#collection_exists?/3","title":"ArangoXEcto.collection_exists?/3","type":"function"},{"doc":"Creates an edge between two modules This can create an edge collection dynamically if no additional fields are required, otherwise an edge schema needs to be specified. The collection name can be passed as an option or is obtained from the provided schema, otherwise it is generated dynamically. Since ArangoDB does not care about the order of the from and two options in anonymous graphs, the order of the from and to attributes used in this function will work either way. Parameters repo - The Ecto repo module to use for queries from - The Ecto Schema struct to use for the from vertex to - The Ecto Schema struct to use for the to vertex opts - Options to use Options Accepts the following options: :edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides collection_name . :fields - The values of the fields to set on the edge. Requires edge to be set otherwise it is ignored. :collection_name - The name of the collection to use. Examples iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 ) % UserUser { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; } Create an edge with a specific edge collection name iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 , collection_name : &quot;friends&quot; ) % Friends { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; } Create a edge schema and use it to create an edge relation defmodule UserPosts do use ArangoXEcto.Edge , from : User , to : Post import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end end iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 , edge : UserPosts , fields : %{ type : &quot;wrote&quot; } ) % UserPosts { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; , from : # Ecto.Association.NotLoaded &lt; association :from is not loaded &gt; , to : # Ecto.Association.NotLoaded &lt; association :to is not loaded &gt; , type : &quot;wrote&quot; }","ref":"ArangoXEcto.html#create_edge/4","title":"ArangoXEcto.create_edge/4","type":"function"},{"doc":"Deletes all edges matching matching the query If the :conditions option is set then those conditions must be true to delete. To just delete one edge do so like any other Ecto Schema struct, i.e. using Ecto.Repo methods. Parameters repo - The Ecto repo module to use for queries from - The Ecto Schema struct to use for the from vertex to - The Ecto Schema struct to use for the to vertex opts - Options to use Options Accepts the following options: :edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides :collection_name . :collection_name - The name of the collection to use. :conditions - A keyword list of conditions to filter for edge deletion Examples Deletes all edges from user1 to user2 iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 ) :ok Deletes all edges from user1 to user2 in specific collection iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 , collection_name : &quot;friends&quot; ) :ok Deletes all edges from user1 to user2 that have matching conditions iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 , conditions : [ type : &quot;best_friend&quot; ] ) :ok","ref":"ArangoXEcto.html#delete_all_edges/4","title":"ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"Generates a edge schema dynamically If a collection name is not provided one will be dynamically generated. The naming convention is the names of the two modules is alphabetical order. E.g. User and Post will combine for a collection name of post_user and an edge module name of PostUser . This order is used to prevent duplicates if the from and to orders are switched. This will create the Ecto Module in the environment dynamically. It will create it under the closest common parent module of the passed modules plus the Edges alias. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.PostUser . Returns the Edge Module name as an atom. Parameters from_module - Ecto Schema Module for the from part of the edge to_module - Ecto Schema Module for the to part of the edge opts - Options passed for module generation Options :collection_name - The name of collection to use instead of generating it Examples iex&gt; ArangoXEcto . edge_module ( MyProject.User , MyProject.Company , [ collection_name : &quot;works_for&quot; ] ) MyProject.WorksFor iex&gt; ArangoXEcto . edge_module ( MyProject.User , MyProject.Company ) MyProject.UsersCompanies","ref":"ArangoXEcto.html#edge_module/3","title":"ArangoXEcto.edge_module/3","type":"function"},{"doc":"Gets an ID from a module and a key Parameters module - Module to get the collection name from key - The _key to use in the id Example iex&gt; ArangoXEcto . get_id_from_module ( User , &quot;123456&quot; ) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_module/2","title":"ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"Gets an ID from a schema struct Parameters struct - The Ecto struct Example If the User schema's collection name is users the following would be: iex&gt; user = % User { id : &quot;123456&quot; } % User { id : &quot;123456&quot; } iex&gt; ArangoXEcto . get_id_from_struct ( user ) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_struct/1","title":"ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"Returns if a Schema is a document schema or not Checks for the presence of the __schema__/1 function on the module and not an edge.","ref":"ArangoXEcto.html#is_document?/1","title":"ArangoXEcto.is_document?/1","type":"function"},{"doc":"Returns if a Schema is an edge or not Checks for the presence of the __edge__/0 function on the module.","ref":"ArangoXEcto.html#is_edge?/1","title":"ArangoXEcto.is_edge?/1","type":"function"},{"doc":"Converts raw output of a query into a struct Transforms string map arguments into atom key map, adds id key and drops _id , _key and _rev keys. Then it creates a struct from filtered arguments using module. If a list of maps are passed then the maps are enumerated over. Parameters maps - List of maps or singular map to convert to a struct module - Module to use for the struct Example iex&gt; { :ok , users } = ArangoXEcto . aql_query ( Repo , &quot;FOR user IN users RETURN user&quot; ) { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] } iex&gt; ArangoXEcto . raw_to_struct ( users , User ) [ % User { id : &quot;12345&quot; , first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } ]","ref":"ArangoXEcto.html#raw_to_struct/2","title":"ArangoXEcto.raw_to_struct/2","type":"function"},{"doc":"Same as schema_type/1 but throws an error on none This is just a shortcut to using is_edge/1 and is_document/1 . If it is neither an error is raised. Examples A real edge schema iex&gt; ArangoXEcto . schema_type! ( MyApp.RealEdge ) :edge Some module that is not an Ecto schema iex&gt; ArangoXEcto . schema_type! ( MyApp.RandomModule ) ** (ArgumentError) Not an Ecto Schema","ref":"ArangoXEcto.html#schema_type!/1","title":"ArangoXEcto.schema_type!/1","type":"function"},{"doc":"Returns the type of a module This is just a shortcut to using is_edge/1 and is_document/1 . If it is neither nil is returned. Examples A real edge schema iex&gt; ArangoXEcto . schema_type ( MyApp.RealEdge ) :edge Some module that is not an Ecto schema iex&gt; ArangoXEcto . schema_type ( MyApp.RandomModule ) nil","ref":"ArangoXEcto.html#schema_type/1","title":"ArangoXEcto.schema_type/1","type":"function"},{"doc":"","ref":"ArangoXEcto.html#t:mod/0","title":"ArangoXEcto.mod/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:query/0","title":"ArangoXEcto.query/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:vars/0","title":"ArangoXEcto.vars/0","type":"type"},{"doc":"Ecto adapter for ArangoDB using ArangoX This implements methods for Ecto.Adapter . These functions should not be accessed directly and should only be used by Ecto. Direct interaction functions are in the ArangoXEcto module.","ref":"ArangoXEcto.Adapter.html","title":"ArangoXEcto.Adapter","type":"module"},{"doc":"Returns true if a connection has been checked out","ref":"ArangoXEcto.Adapter.html#checked_out?/1","title":"ArangoXEcto.Adapter.checked_out?/1","type":"function"},{"doc":"Checks out a connection for the duration of the given function.","ref":"ArangoXEcto.Adapter.html#checkout/3","title":"ArangoXEcto.Adapter.checkout/3","type":"function"},{"doc":"Returns the dumpers for a given type.","ref":"ArangoXEcto.Adapter.html#dumpers/2","title":"ArangoXEcto.Adapter.dumpers/2","type":"function"},{"doc":"Ensure all applications necessary to run the adapter are started","ref":"ArangoXEcto.Adapter.html#ensure_all_started/2","title":"ArangoXEcto.Adapter.ensure_all_started/2","type":"function"},{"doc":"Initialise adapter with config","ref":"ArangoXEcto.Adapter.html#init/1","title":"ArangoXEcto.Adapter.init/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_decimal/1","title":"ArangoXEcto.Adapter.load_decimal/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_float/1","title":"ArangoXEcto.Adapter.load_float/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_integer/1","title":"ArangoXEcto.Adapter.load_integer/1","type":"function"},{"doc":"Returns the loaders of a given type.","ref":"ArangoXEcto.Adapter.html#loaders/2","title":"ArangoXEcto.Adapter.loaders/2","type":"function"},{"doc":"Starts the Agent with an empty list","ref":"ArangoXEcto.Adapter.html#start_link/1","title":"ArangoXEcto.Adapter.start_link/1","type":"function"},{"doc":"Defines an Arango Edge collection as an Ecto Schema Module. Edge modules are dynamically created in the environment if they don't already exist. For more on how edge modules are dynamically generated, please read ArangoXEcto.edge_module/3 . This will define the required fields of an edge ( _from and _to ) and will define the default changeset. Edges utilise Ecto relationships so that the powerful Ecto features can be used. Each Node requires a relationship to be set, either a outgoing/3 or incoming/3 . Behind the scenes this creates an Ecto many to many relationship and generates (or uses the provided) edge module as the intermediary schema. Since the edge collection uses the full _id instead of the _key for the _from and _to fields, once using any of the previously specified relationships a `__id field will be added to structs that will store the value of the _id` field so that relations can be loaded by Ecto. Extending Edge Schemas If you need to add additional fields to an edge, you can do so by creating your own edge module and defining the required fields as well as any additional fields. Luckily there are some helper macros so you don't have to do this manually again. When using a custom edge module, it must be passed to the relationship macros in the nodes using the :edge option, read more about these relationships at ArangoXEcto.Schema . Additionally, the :edge option must also be passed to the ArangoXEcto.create_edge/4 function. A custom schema module must use this module by adding use ArangoXEcto.Edge, from: FromSchema, to: ToSchema . When defining the fields in your schema, make sure to call edge_fields/1 . This will add the _from and _to foreign keys to the schema. It does not have to be before any custom fields but it good convention to do so. A changeset/2 function is automatically defined on the custom schema module but this must be overridden this so that you can cast and validate the custom fields. The edges_changeset/2 method should be called to automatically implement the casting and validation of the _from and _to fields. It does not have to be before any custom field operations but it good convention to do so. Example defmodule MyProject.UserPosts do use ArangoXEcto.Edge , from : User , to : Post import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end end","ref":"ArangoXEcto.Edge.html","title":"ArangoXEcto.Edge","type":"behaviour"},{"doc":"","ref":"ArangoXEcto.Edge.html#c:changeset/2","title":"ArangoXEcto.Edge.changeset/2","type":"callback"},{"doc":"Macro to define the required edge fields i.e. _from and _to . This is required when using a custom edge schema and can be used as below. schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end","ref":"ArangoXEcto.Edge.html#edge_fields/0","title":"ArangoXEcto.Edge.edge_fields/0","type":"macro"},{"doc":"Default changeset for an edge. Casts and requires the _from and _to fields. This will also verify the format of both fields to match that of an Arango id. Any custom changeset should first use this changeset. Direct use of the edges_changeset/2 function is discouraged unless per the use case mentioned above. Example To add a required type field, you could do the following: def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end","ref":"ArangoXEcto.Edge.html#edges_changeset/2","title":"ArangoXEcto.Edge.edges_changeset/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Edge.html#t:t/0","title":"ArangoXEcto.Edge.t/0","type":"type"},{"doc":"Methods for interacting with ArangoDB GeoJSON and geo related functions The methods within this module are really just helpers to generate Geo structs.","ref":"ArangoXEcto.GeoData.html","title":"ArangoXEcto.GeoData","type":"module"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_coordinate/1","title":"ArangoXEcto.GeoData.is_coordinate/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_latitude/1","title":"ArangoXEcto.GeoData.is_latitude/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_longitude/1","title":"ArangoXEcto.GeoData.is_longitude/1","type":"macro"},{"doc":"Generates a Geo linestring","ref":"ArangoXEcto.GeoData.html#linestring/1","title":"ArangoXEcto.GeoData.linestring/1","type":"function"},{"doc":"Generates a Geo multi linestring","ref":"ArangoXEcto.GeoData.html#multi_linestring/1","title":"ArangoXEcto.GeoData.multi_linestring/1","type":"function"},{"doc":"Generates a Geo multi point","ref":"ArangoXEcto.GeoData.html#multi_point/1","title":"ArangoXEcto.GeoData.multi_point/1","type":"function"},{"doc":"Generates a Geo multi polygon","ref":"ArangoXEcto.GeoData.html#multi_polygon/1","title":"ArangoXEcto.GeoData.multi_polygon/1","type":"function"},{"doc":"Generates a Geo point","ref":"ArangoXEcto.GeoData.html#point/2","title":"ArangoXEcto.GeoData.point/2","type":"function"},{"doc":"Generates a Geo polygon","ref":"ArangoXEcto.GeoData.html#polygon/1","title":"ArangoXEcto.GeoData.polygon/1","type":"function"},{"doc":"Sanitizes coordinates to ensure they are valid This function is not automatically applied to Geo constructors and must be applied before hand","ref":"ArangoXEcto.GeoData.html#sanitize/1","title":"ArangoXEcto.GeoData.sanitize/1","type":"function"},{"doc":"","ref":"ArangoXEcto.GeoData.html#t:coordinate/0","title":"ArangoXEcto.GeoData.coordinate/0","type":"type"},{"doc":"Defines Ecto Migrations for ArangoDB NOTE: ArangoXEcto dynamically creates collections for you by default. Depending on your project architecture you may decide to use static migrations instead in which case this module will be useful. Migrations must use this module, otherwise migrations will not work. To do this, replace use Ecto.Migration with use ArangoXEcto.Migration . Since ArangoDB is schemaless, no fields need to be provided, only the collection name. First create a collection struct using the collection/3 function. Then pass the collection struct to the create/1 function. To create indexes it is a similar process using the index/3 function. Collections must be created BEFORE indexes. To drop the collection on a migration down, do the same as creation except use the drop/1 function instead of the create/1 function. Indexes are automatically removed when the collection is removed and cannot be deleted using the drop/1 function. Example defmodule MyProject.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create ( collection ( :users ) ) create ( index ( &quot;users&quot; , [ :email ] ) ) end def down do drop ( collection ( :users ) ) end end","ref":"ArangoXEcto.Migration.html","title":"ArangoXEcto.Migration","type":"module"},{"doc":"Creates a collection struct Used to in functions that perform actions on the database. Accepts a collection type parameter that can either be :document or :edge , otherwise it will raise an error. The default option is :document . Options Accepts an options parameter as the third argument. For available keys please refer to the ArangoDB API doc . Examples iex&gt; collection ( &quot;users&quot; ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 2 ) iex&gt; collection ( &quot;users&quot; , :edge ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 3 ) iex&gt; collection ( &quot;users&quot; , :document , keyOptions : %{ type : :uuid } ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 2 , keyOptions : %{ type : :uuid } )","ref":"ArangoXEcto.Migration.html#collection/3","title":"ArangoXEcto.Migration.collection/3","type":"function"},{"doc":"Creates an object Will create the passed object, either a collection or an index. Examples Create a collection iex&gt; create ( collection ( &quot;users&quot; ) ) :ok Create an index iex&gt; create ( index ( &quot;users&quot; , [ :email ] ) :ok","ref":"ArangoXEcto.Migration.html#create/2","title":"ArangoXEcto.Migration.create/2","type":"function"},{"doc":"Deletes an object Will delete an object passed, can only be a collection, indexes cannot be deleted here. This is because indexes have a randomly generated id and this needs to be known to delete the index, for now this is outside the scope of this project. Example iex&gt; drop ( collection ( &quot;users&quot; ) ) :ok","ref":"ArangoXEcto.Migration.html#drop/2","title":"ArangoXEcto.Migration.drop/2","type":"function"},{"doc":"Creates an edge collection struct Same as passing :edge as the second parameter to collection/3 .","ref":"ArangoXEcto.Migration.html#edge/2","title":"ArangoXEcto.Migration.edge/2","type":"function"},{"doc":"Creates an index struct Default index type is a hash. To change this pass the :type option in options. Options Options only apply to the creation of indexes and has no effect when using the drop/1 function. :type - The type of index to create Accepts: :fulltext , :geo , :hash , :persistent , :skiplist or :ttl :unique - If the index should be unique, defaults to false (hash, persistent &amp; skiplist only) :sparse - If index should be spares, defaults to false (hash, persistent &amp; skiplist only) :deduplication - If duplication of array values should be turned off, defaults to true (hash &amp; skiplist only) :minLength - Minimum character length of words to index (fulltext only) :geoJson - If a geo-spatial index on a location is constructed and geoJson is true, then the order within the array is longitude followed by latitude (geo only) :expireAfter - Time in seconds after a document's creation it should count as expired (ttl only) :name - The name of the index (usefull for phoenix constraints) Examples Create index on email field iex&gt; index ( &quot;users&quot; , [ :email ] ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email ] } Create dual index on email and ph_number fields iex&gt; index ( &quot;users&quot; , [ :email , :ph_number ] ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email , :ph_number ] } Create unique email index iex&gt; index ( &quot;users&quot; , [ :email ] , unique : true ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email ] , unique : true }","ref":"ArangoXEcto.Migration.html#index/3","title":"ArangoXEcto.Migration.index/3","type":"function"},{"doc":"Represent a collection in ArangoDB The attributes in this struct are directly passed to the ArangoDB API for creation. No validation is done on the attributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Collection.html","title":"ArangoXEcto.Migration.Collection","type":"module"},{"doc":"Creates a new Collection struct","ref":"ArangoXEcto.Migration.Collection.html#new/3","title":"ArangoXEcto.Migration.Collection.new/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:collection_option/0","title":"ArangoXEcto.Migration.Collection.collection_option/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:t/0","title":"ArangoXEcto.Migration.Collection.t/0","type":"type"},{"doc":"Represents a collection index in ArangoDB The attributes in this struct are directly passed to the ArangoDB API for creation. No validation is done on the attributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Index.html","title":"ArangoXEcto.Migration.Index","type":"module"},{"doc":"Creates a new Index struct","ref":"ArangoXEcto.Migration.Index.html#new/3","title":"ArangoXEcto.Migration.Index.new/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:index_option/0","title":"ArangoXEcto.Migration.Index.index_option/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:t/0","title":"ArangoXEcto.Migration.Index.t/0","type":"type"},{"doc":"Converts Ecto.Query structs into AQL syntax. This module is copied from https://github.com/ArangoDB-Community/arangodb_ecto/blob/master/lib/arangodb_ecto/query.ex . All credit goes to mpoeter , the original author. Please go check out the original of this file. This is an updated version for Ecto V3","ref":"ArangoXEcto.Query.html","title":"ArangoXEcto.Query","type":"module"},{"doc":"Creates an AQL query to fetch all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#all/1","title":"ArangoXEcto.Query.all/1","type":"function"},{"doc":"Creates an AQL query to delete all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#delete_all/1","title":"ArangoXEcto.Query.delete_all/1","type":"function"},{"doc":"Creates an AQL query to update all entries from the data store matching the given query.","ref":"ArangoXEcto.Query.html#update_all/1","title":"ArangoXEcto.Query.update_all/1","type":"function"},{"doc":"This module is a helper to automatically specify the primary key. The primary key is the Arango _key field but the _id field is also provided. Schema modules should use this module by add use ArangoXEcto.Schema to the module. The only exception to this is if the collection is an edge collection, in that case refer to ArangoXEcto.Edge. Example defmodule MyProject.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name , :string field :last_name , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end","ref":"ArangoXEcto.Schema.html","title":"ArangoXEcto.Schema","type":"module"},{"doc":"Creates a foreign key with correct format Appends _id to the atom","ref":"ArangoXEcto.Schema.html#build_foreign_key/1","title":"ArangoXEcto.Schema.build_foreign_key/1","type":"function"},{"doc":"Defines an incoming relationship Behind the scenes this defines a many to many relationship so that Ecto can load the relationship using the built in functions. The use of this function MUST be accompanied by a outgoing/3 definition in the other target node. This will also define the __id__ field if it is not already defined so that ecto can map the relationship. Example defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string # Will use the automatically generated edge incoming :users , MyProject.User # Will use the UserPosts edge incoming :users , MyProject.User , edge : MyProject.UserPosts end end","ref":"ArangoXEcto.Schema.html#incoming/3","title":"ArangoXEcto.Schema.incoming/3","type":"macro"},{"doc":"Defines indexs for collection dynamic creation The available options can be found in the ArangoXEcto.Migration.Index module. Example To create a generic hash index you don't need to pass the type. indexes [ [ fields : [ :email , :username ] ] ] To create a geoJson index set the type to geo and set geoJson to true. indexes [ [ fields : [ :point ] , type : :geo , geoJson : true ] ] To create a two seperate indexes just supply them seperately. indexes [ [ fields : [ :email ] ] , [ fields : [ :username ] ] ]","ref":"ArangoXEcto.Schema.html#indexes/1","title":"ArangoXEcto.Schema.indexes/1","type":"macro"},{"doc":"Defines an incoming relationship of one object Unlike incoming/3 , this does not create a graph relation and instead places the _id in a field. If the value passed to the name attribute is :user then the foreign key created on this schema will be :user_id and will store the full _id of that user. By storing the full _id , you are still able to perform full AQL queries. This MUST be accompanied by a one_outgoing/3 definition in the other target schema. Behind the scenes this injects the __id__ field to store the _id value and uses the built-in Ecto belongs_to/3 function. Options passed to the opts attribute are passed to the belongs_to/3 definition. Refrain from overriding the :references and :foreign_key attributes unless you know what you are doing. Example defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string one_incoming :user , MyProject.User end end","ref":"ArangoXEcto.Schema.html#one_incoming/3","title":"ArangoXEcto.Schema.one_incoming/3","type":"macro"},{"doc":"Defines an outgoing relationship of one object Unlike outgoing/3 , this does not create a graph relation and instead places the _id in a field in the incoming schema. This MUST be accompanied by a one_incoming/3 definition in the other target schema. Behind the scenes this injects the __id__ field to store the _id value and uses the built-in Ecto has_one/3 function. Options passed to the opts attribute are passed to the has_many/3 definition. Refrain from overriding the :references and :foreign_key attributes unless you know what you are doing. Example defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string one_outgoing :best_post , MyProject.Post end end","ref":"ArangoXEcto.Schema.html#one_outgoing/3","title":"ArangoXEcto.Schema.one_outgoing/3","type":"macro"},{"doc":"Defines collection options for dynamic creation When using dynamic mode for collection creation you may want to add options for when the collection is created. To add options that will be applied you pass them to as a keyword list. The available options can be found in the ArangoXEcto.Migration.Collection module. These options also work in edge collections. Example To specify a UUID as the collection key type you just supply it as the key option type. options [ keyOptions : %{ type : :uuid } ]","ref":"ArangoXEcto.Schema.html#options/1","title":"ArangoXEcto.Schema.options/1","type":"macro"},{"doc":"Defines an outgoing relationship of many objects Behind the scenes this defines a many to many relationship so that Ecto can load the relationship using the built in functions. The use of this function MUST be accompanied by a incoming/3 definition in the other target node. This will also define the __id__ field if it is not already defined so that ecto can map the relationship. Example defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string # Will use the automatically generated edge outgoing :posts , MyProject.Post # Will use the UserPosts edge outgoing :posts , MyProject.Post , edge : MyProject.UserPosts end end","ref":"ArangoXEcto.Schema.html#outgoing/3","title":"ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"Defines a GeoJSON type for use in ecto schemas This will handle conversion of a Geo struct into a map for storage in the database. Use as the type of a field in an Ecto schema. For example: schema &quot;users&quot; do field :location , ArangoXEcto.Types.GeoJSON end","ref":"ArangoXEcto.Types.GeoJSON.html","title":"ArangoXEcto.Types.GeoJSON","type":"module"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"ArangoXEcto.Types.GeoJSON.html#embed_as/1","title":"ArangoXEcto.Types.GeoJSON.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"ArangoXEcto.Types.GeoJSON.html#equal?/2","title":"ArangoXEcto.Types.GeoJSON.equal?/2","type":"function"},{"doc":"Helper methods","ref":"ArangoXEcto.Utils.html","title":"ArangoXEcto.Utils","type":"module"},{"doc":"Get system database connection","ref":"ArangoXEcto.Utils.html#get_system_db/1","title":"ArangoXEcto.Utils.get_system_db/1","type":"function"},{"doc":"Migration task helper functions","ref":"Mix.ArangoXEcto.html","title":"Mix.ArangoXEcto","type":"module"},{"doc":"Creates the database specified in the config","ref":"Mix.ArangoXEcto.html#create_base_database/0","title":"Mix.ArangoXEcto.create_base_database/0","type":"function"},{"doc":"Creates a new database of name","ref":"Mix.ArangoXEcto.html#create_database/1","title":"Mix.ArangoXEcto.create_database/1","type":"function"},{"doc":"Creates a document to store migrations Creates a document with key of the database name. Seperating the migrations into seperate documents allow for more organisation and better debugging.","ref":"Mix.ArangoXEcto.html#create_migration_document/0","title":"Mix.ArangoXEcto.create_migration_document/0","type":"function"},{"doc":"Creates the migrations collection Creates an empty system collection to store migrations.","ref":"Mix.ArangoXEcto.html#create_migrations/0","title":"Mix.ArangoXEcto.create_migrations/0","type":"function"},{"doc":"Gets the default repo's database name","ref":"Mix.ArangoXEcto.html#get_database_name!/0","title":"Mix.ArangoXEcto.get_database_name!/0","type":"function"},{"doc":"Gets the default repo The first in the list of running repos is used.","ref":"Mix.ArangoXEcto.html#get_default_repo!/0","title":"Mix.ArangoXEcto.get_default_repo!/0","type":"function"},{"doc":"Gets all the migrated versions","ref":"Mix.ArangoXEcto.html#migrated_versions/1","title":"Mix.ArangoXEcto.migrated_versions/1","type":"function"},{"doc":"Gets the path to the priv repo folder Will return a full file path to the priv/repo folder.","ref":"Mix.ArangoXEcto.html#path_to_priv_repo/1","title":"Mix.ArangoXEcto.path_to_priv_repo/1","type":"function"},{"doc":"Removes a version from the migrations collection","ref":"Mix.ArangoXEcto.html#remove_version/2","title":"Mix.ArangoXEcto.remove_version/2","type":"function"},{"doc":"Creates a timestamp for the migration file","ref":"Mix.ArangoXEcto.html#timestamp/0","title":"Mix.ArangoXEcto.timestamp/0","type":"function"},{"doc":"Updates the migrated versions in the migration collection","ref":"Mix.ArangoXEcto.html#update_versions/2","title":"Mix.ArangoXEcto.update_versions/2","type":"function"},{"doc":"Specifies a migration version type","ref":"Mix.ArangoXEcto.html#t:version/0","title":"Mix.ArangoXEcto.version/0","type":"type"},{"doc":"Generates a migration. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. Examples mix ecto . gen . migration create_users mix ecto . gen . migration create_users - r Custom.Repo The generated migration filename will be prefixed with the current timestamp in UTC which is used for versioning and ordering. By default, the migration will be generated to the priv/YOUR_REPO/migrations directory of the current application but it can be configured to be any subdirectory of priv by specifying the :priv key under the repository configuration. This generator will automatically open the generated file if you have ECTO_EDITOR set in your environment variable. Command line options -r , --repo - the repo to generate migration for","ref":"Mix.Tasks.Ecto.Gen.Migration.Arango.html","title":"Mix.Tasks.Ecto.Gen.Migration.Arango","type":"task"},{"doc":"Runs Migration/Rollback functions from migration modules","ref":"Mix.Tasks.Ecto.Migrate.Arango.html","title":"Mix.Tasks.Ecto.Migrate.Arango","type":"task"},{"doc":"","ref":"Mix.Tasks.Ecto.Migrate.Arango.html#down/1","title":"Mix.Tasks.Ecto.Migrate.Arango.down/1","type":"function"},{"doc":"Sets up all necessary aspects of the adapter Adds migrations collection in _systems db for migrations and creates the database","ref":"Mix.Tasks.Ecto.Setup.Arango.html","title":"Mix.Tasks.Ecto.Setup.Arango","type":"task"},{"doc":"ArangoXEcto is an all-in-one Arango database adapter for the Elixir Ecto package. It has full support for Graphing , Geo Functions , AQL Integration , amongst other features.","ref":"readme.html","title":"ArangoX Ecto","type":"extras"},{"doc":"ArangoX Ecto Table of Contents About The Project Built With Getting Started Prerequisites Installation Usage Static or Dynamic Basic Usage Repo Setup Schema Setup Migration Setup Raw AQL queries Graph Relations Generation of Edge Modules Graphing in Ecto One-to-One relationships How it works Example More information Further Usage Roadmap Contributing License Contact Acknowledgements","ref":"readme.html#table-of-contents","title":"ArangoX Ecto - Table of Contents","type":"extras"},{"doc":"After playing around with different packages that implemented ArangoDB in Elixir, I found that there wasn't a package that suited my needs. I needed ArangoDB to work with Ecto seamlessly but there was no up-to-date adapter for Ecto available. ArangoX Ecto uses the power of ArangoX to communicate with ArangoDB and Ecto for the API in Elixir. Ecto is integrated with many other packages and can now be used with ArangoDB thanks to this package. From version 1.0.0 onward graph relationships work seamlessly in Ecto. Built With Arangox","ref":"readme.html#about-the-project","title":"ArangoX Ecto - About The Project","type":"extras"},{"doc":"To get the adapter integrated with your project, follow these simple steps. Prerequisites Elixir 1.12.3+ / Erlang OTP 22.2+ (Others may versions may work but this is the oldest it is tested on) Installation Add the following line to your mix dependencies to get started. { :arangox_ecto , &quot;~&gt; 1.1&quot; }","ref":"readme.html#getting-started","title":"ArangoX Ecto - Getting Started","type":"extras"},{"doc":"To connect to the database you need to specify the config values like so: config :my_app , MyApp.Repo , database : &quot;my_db&quot; , endpoints : &quot;http://1.2.3.4:8529&quot; Only database and endpoints are required but there are other available options can be found in the Arangox docs . The database should be setup using $ mix ecto.setup.arango Static or Dynamic In addition to default config variables, the static boolean config option can be passed to force the use of migrations. By default the value is false and collections that don't exist will be created on insert and on query no error will be raised. If set to true , any collections that do not exist on insert or query will result in an error to be raised. Whether dynamic (default) or static is chosen defends on the database design of the project. For a production setup where lots of control is required, it is recommended to have static set to true . Basic Usage Repo Setup To use the adapter in your repo, make sure your repo uses the ArangoXEcto.Adapter module for the adapter. defmodule MyApp.Repo do use Ecto.Repo , otp_app : :my_app , adapter : ArangoXEcto.Adapter end Schema Setup Since ArangoDB uses a slightly different id system, your schema must use the ArangoXEcto.Schema instead of Ecto.Schema . defmodule MyApp.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name , :string field :last_name , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end The options/1 and indexes/1 options can be optionally called inside the schema to set options and indexes to be created in dynamic mode. These options will have no effect in static mode. For example, if you wanted to use a uuid as the key type and create an index on the email the following can be used. defmodule MyApp.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset options [ keyOptions : %{ type : :uuid } ] indexes [ [ fields : [ :email ] ] ] schema &quot;users&quot; do field :email , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end Please refer to the Schema documentation for more information on the available options. Migration Setup Using migrations is only required in static mode If in dynamic mode (default), the adapter will automatically create collections if they don't already exist but there are cases where you might need to use a static system and hence migrations are required. For example, if you needed to create indexes as well, the following would be used. Refer to ArangoXEcto.Migration for more information on usage. defmodule MyApp.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create ( collection ( :users ) ) create ( index ( &quot;users&quot; , [ :email ] ) ) end def down do drop ( collection ( :users ) ) end end Raw AQL queries A lot of the time it is far more efficient to just run a raw AQL query, there's a function for that. ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) This query will return a result such as: { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] } This is awesome functionality, but a lot of the time we will want to resemble a specific struct. This is actually quite easy with the help of the ArangoXEcto.raw_to_struct/2 function. The same query above could be extended to also convert the output: ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) |&gt; case do { :ok , results } -&gt; results |&gt; ArangoXEcto . raw_to_struct ( User ) { :error , _reason } -&gt; [ ] end This will return something like: [ % User { id : &quot;12345&quot; , first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } ] This is clearly a much better representation of the result and can be used in further Ecto methods. Graph Relations The adapter will dynamically create and manage edge collections. Each edge collection will be created as an Ecto schema when they are first used. This will allow for more extensibility through Ecto onto the edges. Generation of Edge Modules The module will be created under the closest common parent module of the passed modules plus the Edges alias. The order of the edge name will always be alphabetical to prevent duplicate edges. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.PostsUsers . This assumes that the edge collection name was generated and not explicitly defined, if it was PostsUsers would be replaced with the camel case of that collection name (i.e. posts_users ). Graphing in Ecto From version 1.0.0 onwards, edges are represented as traditional relationships in Ecto. This approach allows for the simplicity of simple management of edges but the complex querying through AQL. The relationships are represented by an outgoing/3 and incoming/3 in the respective schemas. For example if you wanted to create edges between A and B, with the direction A -&gt; B then A would have outgoing/3 and B would have incoming/3 . This directionality is simply for what is put in the _from and _to fields in the Arango edge collection, if you don't care about directionality, you don't have to worry as much about which schema has outgoing/3 and incoming/3 . One-to-One relationships Additionally, one_outgoing/3 and one_incoming/3 can be used for one-to-one relationships. These do not actually create edges but just store the _id of the target in a field instead. The order of the outgoing &amp; incoming in schemas does matter as a field will be created in the incoming schema. How it works Behind the scenes these outgoing &amp; incoming helper macros are simply wrappers around the Ecto function Ecto.Schema.many_to_many/3 . The one-to-one outgoing &amp; incoming macros are just wrappers around Ecto.Schema.has_one/3 and Ecto.Schema.belongs_to/3 respectively. Hence once setup the regular methods for handling Ecto relationships can be used. In order to delete a specific edge, you can do it exactly as you would any other Ecto struct (since after all it is one) or Ecto relation. Querying of edges can be done either through using an AQL query or by using Ecto methods. Example Let's say you wanted to create an edge between your Post and User schema. You could implement it as follows: defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string # Will use the automatically generated edge outgoing :posts , MyProject.Post # Or use the UserPosts edge # outgoing :posts, MyProject.Post, edge: MyProject.UserPosts end end defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string # Will use the automatically generated edge incoming :users , MyProject.User # Or use the UserPosts edge # incoming :users, MyProject.User, edge: MyProject.UserPosts end end More information To learn about using the schema functions for representing graph relationships and examples, read the docs at ArangoXEcto.Schema . To read more about Edge Schemas and how to extend edge schemas to add additional fields, read the docs on ArangoXEcto.Edge . To learn how to use the helper functions (as well as other useful methods) check out the full documentation . Further Usage For more examples and full documentation, please refer to the Documentation .","ref":"readme.html#usage","title":"ArangoX Ecto - Usage","type":"extras"},{"doc":"See the the roadmap for a list of proposed features (and known issues) planned.","ref":"readme.html#roadmap","title":"ArangoX Ecto - Roadmap","type":"extras"},{"doc":"Contributions are what make the open source community such an amazing place to be learn, inspire, and create. Any contributions you make are greatly appreciated . Checkout the Contributing Guide .","ref":"readme.html#contributing","title":"ArangoX Ecto - Contributing","type":"extras"},{"doc":"Distributed under the Apache 2.0 License. See LICENSE for more information.","ref":"readme.html#license","title":"ArangoX Ecto - License","type":"extras"},{"doc":"Project Home: https://github.com/TomGrozev/arangox_ecto","ref":"readme.html#contact","title":"ArangoX Ecto - Contact","type":"extras"},{"doc":"mpoeter - Wrote the original Ecto Query to AQL code kianmeng - README changes &amp; CI dep change bodbdigr - Fixed AQL query typespec hengestone - Added option to use different migrations directory","ref":"readme.html#acknowledgements","title":"ArangoX Ecto - Acknowledgements","type":"extras"}]